{ *************************************************************************** }
{                                                                             }
{ FloodFill Demo                                                              }
{                                                                             }
{ Copyright (c) 2004 Sergey Mishin <mishinmail@mail.ru>                       }
{                                                                             }
{ This program is free software; you can redistribute it and/or               }
{ modify it under the terms of the GNU General Public                         }
{ License as published by the Free Software Foundation; either                }
{ version 2 of the License, or (at your option) any later version.            }
{                                                                             }
{ This program is distributed in the hope that it will be useful,             }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of              }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           }
{ General Public License for more details.                                    }
{                                                                             }
{ You should have received a copy of the GNU General Public License           }
{ along with this program; see the file COPYING.  If not, write to            }
{ the Free Software Foundation, Inc., 59 Temple Place - Suite 330,            }
{ Boston, MA 02111-1307, USA.                                                 }
{                                                                             }
{ *************************************************************************** }

unit Unit1;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls;

const MaxPixelCount = 32768;

type
  PColor32 = ^TColor32;
  TColor32 = type Longword;
  TColor32Array = array[0..MaxPixelCount - 1] of TColor32;
  PColor32Array = ^TColor32Array;

type
  TForm1 = class(TForm)
    Image1: TImage;
    Button1: TButton;
    OpenDialog1: TOpenDialog;
    ColorDialog1: TColorDialog;
    Button2: TButton;
    Panel1: TPanel;
    PaintBox1: TPaintBox;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    function ColorSwap(WinColor: TColor): TColor32;
    function Color32(WinColor: TColor): TColor32;
    procedure FloodFill(bitmap: TBitmap; X,Y: integer; oldcolor, newcolor: TColor32);
    procedure Fill(X,Y: integer;Button: TMouseButton);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.xfm}
function TForm1.ColorSwap(WinColor: TColor): TColor32;
begin
  Result := $FF000000 or
    TColor32((WinColor and $0000FF) shl  16) or // R
    TColor32( WinColor and $00FF00) or          // G
    TColor32((WinColor and $FF0000) shr 16);    // B
end;

function TForm1.Color32(WinColor: TColor): TColor32;
begin
  WinColor := ColorToRGB(WinColor);
  Result := ColorSwap(WinColor);
end;

procedure TForm1.FloodFill(Bitmap: TBitmap; X,Y: integer; oldcolor, newcolor: TColor32);
var Row: PColor32Array;
begin
      try
           if (X<0) or (Y<0) or (X> Bitmap.Width) or (Y>Bitmap.Height) then exit;
           Row:=Bitmap.ScanLine[Y];
           if (Row[X] = oldcolor) and (Row[X] <> newcolor) then begin Row[X] := newcolor;
               FloodFill(Bitmap, X+1,Y,oldcolor, newcolor);
               FloodFill(Bitmap, X,Y-1,oldcolor, newcolor);
               FloodFill(Bitmap, X-1,Y,oldcolor, newcolor);
               FloodFill(Bitmap, X,Y+1,oldcolor, newcolor);
           end;
      except

      end;

end;

procedure TForm1.Fill(X,Y: integer;Button: TMouseButton);
var oldcolor32, newcolor32 :TColor32;
    Row: PColor32Array;
begin
    Row:=Image1.Picture.Bitmap.Scanline[Y];
    oldcolor32:= Row[X];
    if Button=mbLeft then newcolor32:= Color32(Panel1.Color);
    FloodFill(Image1.Picture.Bitmap, x,y,oldcolor32,newcolor32);
    Image1.Repaint;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
      if OpenDialog1.Execute then
      begin
          Image1.Picture.LoadFromFile(OpenDialog1.FileName);
      end;

end;

procedure TForm1.Button2Click(Sender: TObject);
begin
      if ColorDialog1.Execute then
      begin
          Panel1.Color:=ColorDialog1.Color;
      end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     ColorDialog1.Color:=clYellow;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
     Fill(X,Y,Button);
end;

end.
